/* ***********************************************************
 * This file was automatically generated on 2012-06-28.      *
 *                                                           *
 * If you have a bugfix for this file and want to commit it, *
 * please fix the bug in the generator. You can find a link  *
 * to the generator git on tinkerforge.com                   *
 *************************************************************/

using System;

namespace Tinkerforge
{
	/// <summary>
	///  Device for controlling up to 4 general purpose input/output pins
	/// </summary>
	public class BrickletIO4 : Device 
	{
		private static byte FUNCTION_SET_VALUE = 1;
		private static byte FUNCTION_GET_VALUE = 2;
		private static byte FUNCTION_SET_CONFIGURATION = 3;
		private static byte FUNCTION_GET_CONFIGURATION = 4;
		private static byte FUNCTION_SET_DEBOUNCE_PERIOD = 5;
		private static byte FUNCTION_GET_DEBOUNCE_PERIOD = 6;
		private static byte FUNCTION_SET_INTERRUPT = 7;
		private static byte FUNCTION_GET_INTERRUPT = 8;
		private static byte CALLBACK_INTERRUPT = 9;


		/// <summary>
		///  This callback is triggered whenever a change of the voltage level is detected
		///  on pins where the interrupt was activated with :func:`SetPortInterrupt`.
		///  
		///  The values are a bit mask that specifies which interrupts occurred
		///  and the current value bit mask of the port.
		///  
		///  For example:
		///  
		///  * (1, 1) means that an interrupt on pin 0 occurred and
		///    currently pin 0 is high and pins 1-3 are low.
		///  * (9, 14) means that an interrupt on pins 0 and 3
		///    occurred and currently pin 0 is low and pins 1-3 are high.
		/// </summary>
		public delegate void Interrupt(byte interruptMask, byte valueMask);

		/// <summary>
		///  Creates an object with the unique device ID <c>uid</c>. This object can
		///  then be added to the IP connection.
		/// </summary>
		public BrickletIO4(string uid) : base(uid) 
		{
			this.expectedName = "IO-4 Bricklet";
			this.bindingVersion[0] = 1;
			this.bindingVersion[1] = 0;
			this.bindingVersion[2] = 0;
			messageCallbacks[CALLBACK_INTERRUPT] = new MessageCallback(CallbackInterrupt);
		}

		/// <summary>
		///  Sets the output value (high or low) with a bit mask. The bit mask
		///  is 4 bit long, "true" refers to high and "false" refers to low.
		///  
		///  For example: The value 0b0011 will turn the pins 0-1 high and the
		///  pins 2-3 low.
		///  
		///  <note>
		///    This function does nothing for pins that are configured as input.
		///    Pull up resistors can be switched on with :func:`SetPortConfiguration`.
		///  </note>
		/// </summary>
		public void SetValue(byte valueMask)
		{
			byte[] data_ = new byte[5];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_SET_VALUE, 1, data_);
			LEConverter.To((ushort)5, 2, data_);
			LEConverter.To(valueMask, 4, data_);

			SendRequestNoResponse(data_);
		}

		/// <summary>
		///  Returns a bit mask of the values that are currently measured.
		///  This function works if the pin is configured to input
		///  as well as if it is configured to output.
		/// </summary>
		public byte GetValue()
		{
			byte[] data_ = new byte[4];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_GET_VALUE, 1, data_);
			LEConverter.To((ushort)4, 2, data_);

			byte[] answer;
			SendRequestExpectResponse(data_, FUNCTION_GET_VALUE, out answer);

			return LEConverter.ByteFrom(4, answer);
		}

		/// <summary>
		///  Configures the value and direction of the specified pins. Possible directions
		///  are "i" and "o" for input and output.
		///  
		///  If the direction is configured as output, the value is either high or low
		///  (set as true or false).
		///  
		///  If the direction is configured as input, the value is either pull up or
		///  default (set as true or false).
		///  
		///  For example: 
		///  
		///  * (15, 'i', true) will set all pins of as input pull up.
		///  * (8, 'i', false) will set pin 3 of as input default (floating if nothing is connected).
		///  * (3, 'o', false) will set pins 0 and 1 as output low.
		///  * (4, 'o', true) will set pin 2 of as output high.
		/// </summary>
		public void SetConfiguration(byte pinMask, char direction, bool value)
		{
			byte[] data_ = new byte[7];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_SET_CONFIGURATION, 1, data_);
			LEConverter.To((ushort)7, 2, data_);
			LEConverter.To(pinMask, 4, data_);
			LEConverter.To(direction, 5, data_);
			LEConverter.To(value, 6, data_);

			SendRequestNoResponse(data_);
		}

		/// <summary>
		///  Returns a value bit mask and a direction bit mask.
		///  
		///  For example: A return value of 0b0011 and 0b0101 for
		///  direction and value means that:
		///  
		///  * pin 0 is configured as input pull up,
		///  * pin 1 is configured as input default,
		///  * pin 2 is configured as output high
		///  * and pin 3 is are configured as output low.
		/// </summary>
		public void GetConfiguration(out byte directionMask, out byte valueMask)
		{
			byte[] data_ = new byte[4];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_GET_CONFIGURATION, 1, data_);
			LEConverter.To((ushort)4, 2, data_);

			byte[] answer;
			SendRequestExpectResponse(data_, FUNCTION_GET_CONFIGURATION, out answer);

			directionMask = LEConverter.ByteFrom(4, answer);
			valueMask = LEConverter.ByteFrom(5, answer);
		}

		/// <summary>
		///  Sets the debounce period of the <see cref="Tinkerforge.BrickletIO4.Interrupt"/> callback in ms.
		///  
		///  For example: If you set this value to 100, you will get the interrupt
		///  maximal every 100ms. This is necessary if something that bounces is
		///  connected to the IO-4 Bricklet, such as a button.
		///  
		///  The default value is 100.
		/// </summary>
		public void SetDebouncePeriod(uint debounce)
		{
			byte[] data_ = new byte[8];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_SET_DEBOUNCE_PERIOD, 1, data_);
			LEConverter.To((ushort)8, 2, data_);
			LEConverter.To(debounce, 4, data_);

			SendRequestNoResponse(data_);
		}

		/// <summary>
		///  Returns the debounce period as set by <see cref="Tinkerforge.BrickletIO4.SetDebouncePeriod"/>.
		/// </summary>
		public uint GetDebouncePeriod()
		{
			byte[] data_ = new byte[4];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_GET_DEBOUNCE_PERIOD, 1, data_);
			LEConverter.To((ushort)4, 2, data_);

			byte[] answer;
			SendRequestExpectResponse(data_, FUNCTION_GET_DEBOUNCE_PERIOD, out answer);

			return LEConverter.UIntFrom(4, answer);
		}

		/// <summary>
		///  Sets the pins on which an interrupt is activated with a bit mask. 
		///  Interrupts are triggered on changes of the voltage level of the pin,
		///  i.e. changes from high to low and low to high.
		///  
		///  For example: An interrupt bit mask of 9 will enable the interrupt for 
		///  pins 0 and 3.
		///  
		///  The interrupt is delivered with the callback <see cref="Tinkerforge.BrickletIO4.Interrupt"/>.
		/// </summary>
		public void SetInterrupt(byte interruptMask)
		{
			byte[] data_ = new byte[5];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_SET_INTERRUPT, 1, data_);
			LEConverter.To((ushort)5, 2, data_);
			LEConverter.To(interruptMask, 4, data_);

			SendRequestNoResponse(data_);
		}

		/// <summary>
		///  Returns the interrupt bit mask as set by :func:`SetPortInterrupt`.
		/// </summary>
		public byte GetInterrupt()
		{
			byte[] data_ = new byte[4];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_GET_INTERRUPT, 1, data_);
			LEConverter.To((ushort)4, 2, data_);

			byte[] answer;
			SendRequestExpectResponse(data_, FUNCTION_GET_INTERRUPT, out answer);

			return LEConverter.ByteFrom(4, answer);
		}

		/// <summary>
		///  Obsolete. Use overloaded version instead that returns the result.
		/// </summary>
		[Obsolete()]
		public void GetValue(out byte valueMask)
		{
			valueMask = GetValue();
		}

		/// <summary>
		///  Obsolete. Use overloaded version instead that returns the result.
		/// </summary>
		[Obsolete()]
		public void GetDebouncePeriod(out uint debounce)
		{
			debounce = GetDebouncePeriod();
		}

		/// <summary>
		///  Obsolete. Use overloaded version instead that returns the result.
		/// </summary>
		[Obsolete()]
		public void GetInterrupt(out byte interruptMask)
		{
			interruptMask = GetInterrupt();
		}

		internal int CallbackInterrupt(byte[] data_)
		{
			byte interruptMask = LEConverter.ByteFrom(4, data_);
			byte valueMask = LEConverter.ByteFrom(5, data_);

			((Interrupt)callbacks[CALLBACK_INTERRUPT])(interruptMask, valueMask);
			return 6;
		}

		/// <summary>
		///  Registers a callback function.
		/// </summary>
		public void RegisterCallback(System.Delegate d)
		{
			if(d.GetType() == typeof(Interrupt))
			{
				callbacks[CALLBACK_INTERRUPT] = d;
			}
		}
	}
}
